<?php
require '../spotify/spotify.php';

if (isset($_GET['logout'])) {
    $_SESSION['accessToken'] = '';
    $_SESSION['refreshToken'] = '';
}

$_SESSION['accessToken'] = isset($_SESSION['accessToken']) ? $_SESSION['accessToken'] : '';
$_SESSION['refreshToken'] = isset($_SESSION['refreshToken']) ? $_SESSION['refreshToken'] : '';

if (!$_SESSION['accessToken']) {
    header('Location: auth.php');
    die();
}

$session = new SpotifyWebAPI\Session($CLIENT_ID, $CLIENT_SECRET, $REDIRECT_URI);

// Use previously requested tokens fetched from session
if ($_SESSION['accessToken']) {
    $session->setAccessToken($_SESSION['accessToken']);
    $session->setRefreshToken($_SESSION['refreshToken']);
} else {
    // Or request a new access token
    $session->refreshAccessToken($_SESSION['refreshToken']);
}

$options = [
  'scope' => $SPOTIFY_REQUIRED_SCOPES,
  'auto_refresh' => true,
];

$api = new SpotifyWebAPI\SpotifyWebAPI($options, $session);

// Save new tokens, they might have been updated
$_SESSION['accessToken'] = $session->getAccessToken();
$_SESSION['refreshToken'] = $session->getRefreshToken();

// ################################################################

$user = $api->me();

$templateData = [
  'user' => $user,
  'playlists' => [],
  'topArtists' => [],

  'infos' => [],
  'errors' => [],
];

$generatePlaylistFromPlaylist = isset($_GET['generate-playlist-from-playlist']);
$generatePlaylistFromArtists = isset($_GET['generate-playlist-top-artists']);

if ($generatePlaylistFromPlaylist) {
    $selectedPlaylists = $_GET['selected-playlist'];
    error_log('Will create new playlist with recommendations from playlists: '.join(', ', $selectedPlaylists).'".');
    $recommendedTrackIds = [];

    $shouldFilterByArtists = isset($_GET['check-playlist-filter-artists']);
    error_log('Filter by artists in source playlists: '.($shouldFilterByArtists ? 'yes' : 'no'));
    $artistsInSourcePlaylists = [];

    foreach ($selectedPlaylists as $selectedPlaylistId) {
        $playlist = $api->getPlaylist($selectedPlaylistId);
        error_log('Will get recommendations from tracks in playlist "'.$playlist->name.'".');
        $playlistTracks = $playlist->tracks->items;
        error_log('Found '.count($playlistTracks).' tracks.');
        foreach ($playlistTracks as $track) {
            $artists = [];
            foreach ($track->track->artists as $artist) {
                $artists[] = $artist->name;
                $artistsInSourcePlaylists[$artist->id] = 1;
            }
            error_log('Track "'.$track->track->name.'" ('.join(', ', $artists).')');

            $recommendations = $api->getRecommendations([
              'seed_tracks' => [$track->track->id],
              'limit' => 40
          ]);
            error_log(' -> Got '.count($recommendations->tracks).' recommendatations.');
            foreach ($recommendations->tracks as $recommendedTrack) {
                $recommendedTrackIds[$recommendedTrack->id] = $recommendedTrack;
            }
        }
    }

    if ($shouldFilterByArtists) {
        $artistsInSourcePlaylists = array_keys($artistsInSourcePlaylists);
        error_log('Filter by artists found in source playlists ('.count($artistsInSourcePlaylists).')');

        $filteredTrackIds = [];
        foreach ($recommendedTrackIds as $trackId => $track) {
            $trackArtistInWantedArtists = false;
            foreach ($track->artists as $trackArtist) {
                if (\in_array($trackArtist->id, $artistsInSourcePlaylists)) {
                    $filteredTrackIds[$track->id] = 1;
                }
            }
        }
        $recommendedTrackIds = $filteredTrackIds;
    }

    $recommendedTrackIds = array_keys($recommendedTrackIds);
    error_log('=> Got '.count($recommendedTrackIds).' unique recommendatations for this playlist.');
    shuffle($recommendedTrackIds);
    $pickedTrackIds = array_slice($recommendedTrackIds, 0, 50);
    error_log('Keep '.count($pickedTrackIds).' tracks.');

    $now = new DateTime();
    $newPlaylistName = 'Fresh playlist ['.$now->format('Y-m-d H:i:s').']';
    error_log('Create new playlist "'.$newPlaylistName.'".');
    $newPlaylist = $api->createPlaylist([
        'name' => $newPlaylistName,
    ]);

    $api->replacePlaylistTracks($newPlaylist->id, $pickedTrackIds);
    $playlistLink = '<a href="'.$newPlaylist->external_urls->spotify.'">'.$newPlaylistName.'</a>';
    $templateData['infos'][] = 'Ok created new playlist with '.count($pickedTrackIds).' tracks: '.$playlistLink;
} elseif ($generatePlaylistFromArtists) {
    $selectedArtists = $_GET['selected-artist'];
    error_log('Will create new playlist with filterted recommendations from artists: '.join(', ', $selectedArtists).'".');
    $recommendedTrackIds = [];

    $shouldFilterByArtists = isset($_GET['check-top-artist-filter-artists']);
    error_log('Filter by artists in selection: '.($shouldFilterByArtists ? 'yes' : 'no'));

    foreach ($selectedArtists as $selectedArtist) {
        error_log('Get recommendations for artist '.$selectedArtist.'');
        $recommendations = $api->getRecommendations([
            'seed_artists' => $selectedArtist,
            'limit' => 100
        ]);
        error_log(' -> Got '.count($recommendations->tracks).' recommendatations.');

        // Filter by artist if requested, remove duplicates
        foreach ($recommendations->tracks as $recommendedTrack) {
            if ($shouldFilterByArtists) {
                $trackArtistInWantedArtists = false;
                foreach ($recommendedTrack->artists as $trackArtist) {
                    if (\in_array($trackArtist->id, $selectedArtists)) {
                        $recommendedTrackIds[$recommendedTrack->id] = 1;
                    }
                }
            } else {
                $recommendedTrackIds[$recommendedTrack->id] = 1;
            }
        }
    }

    $recommendedTrackIds = array_keys($recommendedTrackIds);
    error_log('=> Got '.count($recommendedTrackIds).' unique recommendatations.');
    shuffle($recommendedTrackIds);
    $pickedTrackIds = array_slice($recommendedTrackIds, 0, 50);
    error_log('Keep '.count($pickedTrackIds).' random tracks.');

    $now = new DateTime();
    $newPlaylistName = 'Fresh playlist ['.$now->format('Y-m-d H:i:s').']';
    error_log('Create new playlist "'.$newPlaylistName.'".');
    $newPlaylist = $api->createPlaylist([
        'name' => $newPlaylistName,
    ]);

    $api->replacePlaylistTracks($newPlaylist->id, $pickedTrackIds);
    $playlistLink = '<a href="'.$newPlaylist->external_urls->spotify.'">'.$newPlaylistName.'</a>';
    $templateData['infos'][] = 'Ok created new playlist with '.count($pickedTrackIds).' tracks: '.$playlistLink;
} else {
    $templateData['topArtists'] = $api->getMyTop('artists', ['limit' => 24, 'time_range' => 'short_term'])->items;
    $templateData['playlists'] = $api->getUserPlaylists($user->id, ['limit' => 24])->items;
}

require '../template.php';
